#include "BMP.h"
#include "GRAPH.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

void background(Bmp* this, Color c) {
	fillRect(this, c, 0, 0, this->width, this->height);
}

void fillRect(Bmp* this, Color c, int x, int y, int w, int h) {
	if (h >= w) {
		for (int i = 0; i <= h; i++) {
			for (int j = 0; j <= w; j++) {
				setBmpPoint(this, c, j+x, i+y); } }
	} else {
		for (int i = 0; i <= w; i++) {
			for (int j = 0; j <= h; j++) {
				setBmpPoint(this, c, i+x, j+y); } }
	}
}

/* Copied from skript.pdf *************************************************/
void swapValues(int* x,int* y){
	int z= *x;
	*x= *y;
	*y = z;
}

void drawLine(Bmp* this, Color c, int fromX, int fromY, int toX, int toY) {
	int xDiv = abs(toX-fromX);
	int yDiv = abs(toY-fromY);

	if (fromX==toX && fromY==toY){
		setBmpPoint(this,c,fromX,fromY);
	}else if (xDiv>yDiv){
		if (fromX>toX){
			swapValues(&fromY,&toY);
			swapValues(&fromX,&toX);
		}
		double steigung= ((double)toY-fromY)/(toX-fromX);
		for(int x=fromX;x<=toX;x++){
			setBmpPoint(this,c,x,fromY+(int)(x-fromX)*steigung);
		}
	}else{
		if (fromY>toY){
			swapValues(&fromY,&toY);
			swapValues(&fromX,&toX);
		}
		double steigung= ((double)toX-fromX)/(toY-fromY);
		for(int y=fromY;y<=toY;y++){
			setBmpPoint(this,c,fromX+(int)(y-fromY)*steigung,y);
		}
	}
}
/**************************************************************************/

Bmp* drawGraph(double (f)(double), double minX, double maxX) {
	int sX = 5;
	int sY = 5;

	int width;
	if (minX >= 0 && maxX >= 0) {
		width = maxX * 2 * sX;
	} else if (minX <= 0 && maxX <= 0) {
		width = abs(minX) * 2 * sX;
	} else {
		width = (maxX - minX) * sX;
	}

	int height = abs((int)f(minX)) * sY;
	for (int i = minX; i <= maxX; i++) {
		if (abs((int)f(i)) * sY > height) {
			height = abs((int)f(i)) * sY;
		}
	}

	if (height > width) {
		width = height;
	} else {
		height = width;
	}

	Bmp* this = newBmp(width, height);
	background(this, WHITE);

	drawLine(this, BLACK, width/2, 0, width/2, height);
	drawLine(this, BLACK, 0, height/2, width, height/2);

	srand(time(NULL));
	Color c = {rand() % 256, rand() % 256, rand() % 256};
	for (int i = minX; i < maxX; i++) {
		drawLine(this, c, width/2 + i*sX, height/2 + f(i)*sY, width/2 + (i+1)*sX, height/2 + f(i+1)*sY);
	}

	return this;
}

double ab(double (f)(double), double x) {
	return ((f(x+0.1)-f(x)) / ((x+0.1) - x));
}

double abN(double (f)(double), double x, unsigned int n) {
	double ab(double x) {
		return ((f(x+0.1)-f(x)) / ((x+0.1) - x));
	}

	return n==0 ? f(x) : abN(ab, x, n-1);
}

Bmp* drawGraphWithDiffs(double (f)(double), double minX, double maxX, unsigned int numberOfDiffs) {
	int sX = 5;
	int sY = 5;

	int width;
	if (minX >= 0 && maxX >= 0) {
		width = maxX * 2 * sX;
	} else if (minX <= 0 && maxX <= 0) {
		width = abs(minX) * 2 * sX;
	} else {
		width = (maxX - minX) * sX;
	}

	int height = abs((int)abN(f,minX,0)) * sY;
	for (int i = 0; i <= numberOfDiffs; i++) {
		for (int j = minX; j <= maxX; j++) {
			if (abs((int)abN(f,j,i)) * sY > height) {
				height = abs((int)abN(f,j,i)) * sY;
			}
		}
	}

	if (height > width) {
		width = height;
	} else {
		height = width;
	}

	Bmp* this = newBmp(width, height);
	background(this, WHITE);

	drawLine(this, BLACK, width/2, 0, width/2, height);
	drawLine(this, BLACK, 0, height/2, width, height/2);

	srand(time(NULL));
	for (int i = 0; i <= numberOfDiffs; i++) {
		Color c = {rand() % 256, rand() % 256, rand() % 256};

		for (int j = minX; j < maxX; j++) {
			drawLine(this, c, width/2 + j*sX, height/2 + abN(f,j,i)*sY, width/2 + (j+1)*sX, height/2 + abN(f,j+1,i)*sY);
		}
	}

	return this;
}
